"""The auxoutput commands module.

These commands are used in the following models:
AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUXoutput[n]:SOURce {AFLag|BFLag|CFLag|DFLag|TIMer}
    - AUXoutput[n]:SOURce:CMAPping <channel>
    - AUXoutput[n]:SOURce:CMAPping?
    - AUXoutput[n]:SOURce?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AuxoutputItemSourceCmapping(SCPICmdWrite, SCPICmdRead):
    """The ``AUXoutput[n]:SOURce:CMAPping`` command.

    Description:
        - This command sets or returns the Auxiliary Output channel mapping.

    Usage:
        - Using the ``.query()`` method will send the ``AUXoutput[n]:SOURce:CMAPping?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXoutput[n]:SOURce:CMAPping?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXoutput[n]:SOURce:CMAPping value``
          command.

    SCPI Syntax:
        ```
        - AUXoutput[n]:SOURce:CMAPping <channel>
        - AUXoutput[n]:SOURce:CMAPping?
        ```
    """


class AuxoutputItemSource(SCPICmdWrite, SCPICmdRead):
    """The ``AUXoutput[n]:SOURce`` command.

    Description:
        - This command sets or returns the signal source for the specified Auxiliary Output
          connector.

    Usage:
        - Using the ``.query()`` method will send the ``AUXoutput[n]:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXoutput[n]:SOURce?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXoutput[n]:SOURce value`` command.

    SCPI Syntax:
        ```
        - AUXoutput[n]:SOURce {AFLag|BFLag|CFLag|DFLag|TIMer}
        - AUXoutput[n]:SOURce?
        ```

    Properties:
        - ``.cmapping``: The ``AUXoutput[n]:SOURce:CMAPping`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._cmapping = AuxoutputItemSourceCmapping(device, f"{self._cmd_syntax}:CMAPping")

    @property
    def cmapping(self) -> AuxoutputItemSourceCmapping:
        """Return the ``AUXoutput[n]:SOURce:CMAPping`` command.

        Description:
            - This command sets or returns the Auxiliary Output channel mapping.

        Usage:
            - Using the ``.query()`` method will send the ``AUXoutput[n]:SOURce:CMAPping?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXoutput[n]:SOURce:CMAPping?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AUXoutput[n]:SOURce:CMAPping value`` command.

        SCPI Syntax:
            ```
            - AUXoutput[n]:SOURce:CMAPping <channel>
            - AUXoutput[n]:SOURce:CMAPping?
            ```
        """
        return self._cmapping


class AuxoutputItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``AUXoutput[n]`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXoutput[n]?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXoutput[n]?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.source``: The ``AUXoutput[n]:SOURce`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "AUXoutput[n]"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._source = AuxoutputItemSource(device, f"{self._cmd_syntax}:SOURce")

    @property
    def source(self) -> AuxoutputItemSource:
        """Return the ``AUXoutput[n]:SOURce`` command.

        Description:
            - This command sets or returns the signal source for the specified Auxiliary Output
              connector.

        Usage:
            - Using the ``.query()`` method will send the ``AUXoutput[n]:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXoutput[n]:SOURce?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXoutput[n]:SOURce value``
              command.

        SCPI Syntax:
            ```
            - AUXoutput[n]:SOURce {AFLag|BFLag|CFLag|DFLag|TIMer}
            - AUXoutput[n]:SOURce?
            ```

        Sub-properties:
            - ``.cmapping``: The ``AUXoutput[n]:SOURce:CMAPping`` command.
        """
        return self._source



"""The dese commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC,
DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K,
MSO5KB, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DESE <NR1>
    - DESE?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Dese(SCPICmdWrite, SCPICmdRead):
    """The ``DESE`` command.

    Description:
        - This command sets and queries the bits in the Device Event Status Enable Register (DESER).
          The DESER is the mask that determines whether events are reported to the Standard Event
          Status Register (SESR), and entered into the Event Queue. For a more detailed discussion
          of the use of these registers, see Registers.

    Usage:
        - Using the ``.query()`` method will send the ``DESE?`` query.
        - Using the ``.verify(value)`` method will send the ``DESE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DESE value`` command.

    SCPI Syntax:
        ```
        - DESE <NR1>
        - DESE?
        ```

    Info:
        - ``<NR1>`` The binary bits of the DESER are set according to this value, which ranges from
          1 through 255. For example, ``DESE 209`` sets the DESER to the binary value 11010001 (that
          is, the most significant bit in the register is set to 1, the next most significant bit to
          1, the next bit to 0, etc.).
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DESE") -> None:
        super().__init__(device, cmd_syntax)



# pylint: disable=line-too-long
"""The data commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DATa {INIT|SNAp}
    - DATa:DESTination REF<x>
    - DATa:DESTination?
    - DATa:ENCdg {ASCIi|FAStest|RIBinary|RPBinary|SRIbinary|SRPbinary|FPbinary|SFPbinary}
    - DATa:ENCdg?
    - DATa:SOUrce {CH<x>|MATH|REF<x>|D<x>|DIGital|RF_AMPlitude|RF_FREQuency|RF_PHASe|RF_NORMal|RF_AVErage|RF_MAXHold|RF_MINHold}
    - DATa:SOUrce?
    - DATa:STARt <NR1>
    - DATa:STARt?
    - DATa:STOP <NR1>
    - DATa:STOP?
    - DATa:WIDth <NR1>
    - DATa:WIDth?
    - DATa?
    ```
"""  # noqa: E501

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class DataWidth(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:WIDth`` command.

    Description:
        - This command specifies the width, in bytes per point, for waveform data transferred from
          the instrument via the CURVe? query. (This command is synonymous with
          ``WFMOutpre:BYT_Nr``.)

    Usage:
        - Using the ``.query()`` method will send the ``DATa:WIDth?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:WIDth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:WIDth value`` command.

    SCPI Syntax:
        ```
        - DATa:WIDth <NR1>
        - DATa:WIDth?
        ```

    Info:
        - ``<NR1>`` is an integer that indicates the number of bytes per point for the outgoing
          waveform data when queried using the CURVe? command. For analog channels, NR1 can be 1 or
          2.
    """


class DataStop(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:STOP`` command.

    Description:
        - This command sets or queries the last data point that will be transferred when using the
          CURVE? query. This command allows for the transfer of partial waveforms to the controller.
          Changes to the record length value are not automatically reflected in the ``data:stop``
          value. As record length is varied, the ``DATa:STOP`` value must be explicitly changed to
          ensure the entire record is transmitted. In other words, curve results will not
          automatically and correctly reflect increases in record length if the distance from
          ``DATa:STARt`` to ``DATa:STOP`` stays smaller than the increased record length.

    Usage:
        - Using the ``.query()`` method will send the ``DATa:STOP?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:STOP?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:STOP value`` command.

    SCPI Syntax:
        ```
        - DATa:STOP <NR1>
        - DATa:STOP?
        ```

    Info:
        - ``<NR1>`` is the last data point that will be transferred, which ranges from 1 to the
          record length. If <NR1> is greater than the record length, then data will be transferred
          up to the record length. If both ``DATa:STARt`` and ``DATa:STOP`` are greater than the
          record length, the last data point in the record is returned.
    """


class DataStart(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:STARt`` command.

    Description:
        - This command sets or queries the starting data point for waveform transfer. This command
          allows for the transfer of partial waveforms to and from the instrument.

    Usage:
        - Using the ``.query()`` method will send the ``DATa:STARt?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:STARt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:STARt value`` command.

    SCPI Syntax:
        ```
        - DATa:STARt <NR1>
        - DATa:STARt?
        ```

    Info:
        - ``<NR1>`` is the first data point that will be transferred, which ranges from 1 to the
          record length. Data will be transferred from <NR1> to ``DATa:STOP`` or the record length,
          whichever is less. If <NR1> is greater than the record length, the last data point in the
          record is transferred.
        - ``DATa:STARt`` and ``DATa:STOP`` are order independent. When ``DATa:STOP`` is greater than
          ``DATa:STARt``, the values will be swapped internally for the CURVE? query.
    """


class DataSource(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:SOUrce`` command.

    Description:
        - This command specifies the source waveform to be transferred from the oscilloscope using
          the CURVe? query. The valid waveform sources are CH1-CH4, MATH, REF1-REF4, D0-D15,
          DIGital, ``RF_AMPlitude``, ``RF_FREQuency``, ``RF_PHASe``, ``RF_NORMal``, ``RF_AVErage``,
          ``RF_MAXHold``, and ``RF_MINHold``. Setting ``DATa:SOUrce`` automatically constrains the
          following to valid values for the specified source waveform: ``WFMOutpre:BYT_Nr``,
          ``WFMOutpre:BIT_Nr`` and ``WFMOutpre:BN_Fmt``.

    Usage:
        - Using the ``.query()`` method will send the ``DATa:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:SOUrce value`` command.

    SCPI Syntax:
        ```
        - DATa:SOUrce {CH<x>|MATH|REF<x>|D<x>|DIGital|RF_AMPlitude|RF_FREQuency|RF_PHASe|RF_NORMal|RF_AVErage|RF_MAXHold|RF_MINHold}
        - DATa:SOUrce?
        ```

    Info:
        - ``CH<x>`` specifies which analog channel waveform data will be transferred from the
          oscilloscope to the controller, channels 1 through 4.
        - ``MATH`` specifies that the Math waveform data will be transferred from the oscilloscope
          to the controller.
        - ``REF<x>`` specifies which Reference waveform data will be transferred from the
          oscilloscope to the controller.
        - ``D<x>`` specifies which digital channel waveform data will be transferred from the
          oscilloscope to the controller. (Requires installation of option 3-MSO.).
        - ``DIGital`` specifies that the Digital Collection waveform data will be transferred from
          the oscilloscope to the controller. (Requires installation of option 3-MSO.).
        - ``RF_NORMal`` specify that the RF data will be transferred from the oscilloscope to the
          controller.
        - ``RF_AVErage`` specify that the RF data will be transferred from the oscilloscope to the
          controller.
        - ``RF_MAXHold`` specify that the RF data will be transferred from the oscilloscope to the
          controller.
        - ``RF_MINHold`` specify that the RF data will be transferred from the oscilloscope to the
          controller.
    """  # noqa: E501


class DataEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:ENCdg`` command.

    Description:
        - This command specifies the encoding format for outgoing waveform data. This command is
          equivalent to setting ``WFMOUTPRE:ENCDG``, ``WFMOUTPRE:BN_FMT``, and ``WFMOUTPRE:BYT_OR``.
          Setting the ``DATa:ENGdg`` value causes the corresponding WFMOutpre values to be updated.

    Usage:
        - Using the ``.query()`` method will send the ``DATa:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:ENCdg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:ENCdg value`` command.

    SCPI Syntax:
        ```
        - DATa:ENCdg {ASCIi|FAStest|RIBinary|RPBinary|SRIbinary|SRPbinary|FPbinary|SFPbinary}
        - DATa:ENCdg?
        ```

    Info:
        - ``ASCIi`` specifies to use ASCII encoding for the waveform data queried using the CURVe?
          query. Data values are returned as signed decimal integers for analog channel and RF time
          domain data, 4-byte floating point values for RF frequency domain data, or hexadecimal
          values for Digital Collection data with 4 or 8 bytes per point. The maximum number of
          ASCII data points that can be queried using the CURVe? query is 1 million points. If more
          than 1 million points are desired, you must use one of the binary encodings. If ASCII is
          the value, then ``BN_Fmt`` and ``BYT_Or`` are ignored.
        - ``FAStest`` specifies the encoding which results in the fastest waveform data transfer
          rate. This sets the following: ``WFMOutpre:ENCdg BINary``, ``WFMOutpre:BIN_Fmt RI`` and
          ``WFMOutpre:BYT_Or MSB``.
        - ``RIBinary`` specifies the signed integer data point format, with the most significant
          byte transferred first.
        - ``DATa:WIDTH`` is set to 1, the range is from -128 through 127. When ``DATa:WIDTH`` is 2,
          the range is from -32,768 through 32,768. Center screen is 0 (zero). The upper limit is
          the top of the screen and the lower limit is the bottom of the screen. The default
          encoding is RIBINARY. This sets the following: ``WFMOutpre:ENCdg BINary``,
          ``WFMOutpre:BN_Fmt RI`` and ``WFMOutpre:BYT_Or MSB``.
        - ``RPBinary`` specifies the positive integer data-point representation, with the most
          significant byte transferred first.
        - ``BYT_Nr`` is 1, the range of data values is 0 through 255. When ``BYT_Nr`` is 2, the
          range of data values is 0 to 65,535. The center of the screen is 127 for 1-byte data and
          is 32768 for 2-byte data. The upper limit is the top of the screen and the lower limit is
          the bottom of the screen. This sets the following:
          ``:WFMOutpre:ENCdg BINary``,``:WFMOutpre:BN_Fmt RP`` and ``WFMOutpre:BYT_Or MSB``.
        - ``SRIbinary`` specifies the signed integer format. It is the same as RIBinary except that
          the byte order is swapped, meaning that the least significant byte is transferred first.
          This sets the following: ``WFMOutpre:ENCdg BINary``, ``WFMOutpre:BIN_Fmt RI`` and
          ``WFMOutpre:BYT_Or LSB``.
        - ``SRPbinary`` specifies the positive integer format. It is the same as RPBinary except
          that the byte order is swapped, meaning that the least significant byte is transferred
          first. This sets the following: ``WFMOutpre:ENCdg BINary``, ``WFMOutpre:BN_Fmt RP`` and
          ``WFMOutpre:BYT_Or LSB``.
        - ``FPbinary`` specifies floating point binary. It automatically forces ``DATa:WIDTH`` to 4
          and ``BYT_OR`` to MSB (most significant byte transmitted first).
        - ``SFPbinary`` specifies floating point binary. It automatically forces ``DATa:WIDTH`` to 4
          and ``BYT_OR`` to LSB (least significant byte transmitted first).
    """


class DataDestination(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:DESTination`` command.

    Description:
        - This command specifies the reference memory location (REF1-4) for storing waveform data
          transferred into the oscilloscope using the CURVE command.

    Usage:
        - Using the ``.query()`` method will send the ``DATa:DESTination?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:DESTination?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:DESTination value`` command.

    SCPI Syntax:
        ```
        - DATa:DESTination REF<x>
        - DATa:DESTination?
        ```

    Info:
        - ``REF<x>`` is the reference location where the waveform will be stored.
    """


class Data(SCPICmdWrite, SCPICmdRead):
    """The ``DATa`` command.

    Description:
        - This command sets or queries the format and location of the waveform data that is
          transferred with the CURVE command.

    Usage:
        - Using the ``.query()`` method will send the ``DATa?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa value`` command.

    SCPI Syntax:
        ```
        - DATa {INIT|SNAp}
        - DATa?
        ```

    Info:
        - ``INIT`` initializes the waveform data parameters to their factory defaults except for
          ``DATa:STOP``, which isset to the current acquisition record length.
        - ``SNAp`` Sets ``DATa:STARt`` and ``DATa:STOP`` to match the current waveform cursor
          positions of WAVEVIEW1 CURSOR1 if these waveform cursors are currently on. If these
          waveform cursors are not on when the ``DATa SNAp`` command is sent, it is silently ignored
          and ``DATa:STARt`` and ``:STOP`` remain unchanged.

    Properties:
        - ``.destination``: The ``DATa:DESTination`` command.
        - ``.encdg``: The ``DATa:ENCdg`` command.
        - ``.source``: The ``DATa:SOUrce`` command.
        - ``.start``: The ``DATa:STARt`` command.
        - ``.stop``: The ``DATa:STOP`` command.
        - ``.width``: The ``DATa:WIDth`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DATa") -> None:
        super().__init__(device, cmd_syntax)
        self._destination = DataDestination(device, f"{self._cmd_syntax}:DESTination")
        self._encdg = DataEncdg(device, f"{self._cmd_syntax}:ENCdg")
        self._source = DataSource(device, f"{self._cmd_syntax}:SOUrce")
        self._start = DataStart(device, f"{self._cmd_syntax}:STARt")
        self._stop = DataStop(device, f"{self._cmd_syntax}:STOP")
        self._width = DataWidth(device, f"{self._cmd_syntax}:WIDth")

    @property
    def destination(self) -> DataDestination:
        """Return the ``DATa:DESTination`` command.

        Description:
            - This command specifies the reference memory location (REF1-4) for storing waveform
              data transferred into the oscilloscope using the CURVE command.

        Usage:
            - Using the ``.query()`` method will send the ``DATa:DESTination?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:DESTination?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:DESTination value`` command.

        SCPI Syntax:
            ```
            - DATa:DESTination REF<x>
            - DATa:DESTination?
            ```

        Info:
            - ``REF<x>`` is the reference location where the waveform will be stored.
        """
        return self._destination

    @property
    def encdg(self) -> DataEncdg:
        """Return the ``DATa:ENCdg`` command.

        Description:
            - This command specifies the encoding format for outgoing waveform data. This command is
              equivalent to setting ``WFMOUTPRE:ENCDG``, ``WFMOUTPRE:BN_FMT``, and
              ``WFMOUTPRE:BYT_OR``. Setting the ``DATa:ENGdg`` value causes the corresponding
              WFMOutpre values to be updated.

        Usage:
            - Using the ``.query()`` method will send the ``DATa:ENCdg?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:ENCdg?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:ENCdg value`` command.

        SCPI Syntax:
            ```
            - DATa:ENCdg {ASCIi|FAStest|RIBinary|RPBinary|SRIbinary|SRPbinary|FPbinary|SFPbinary}
            - DATa:ENCdg?
            ```

        Info:
            - ``ASCIi`` specifies to use ASCII encoding for the waveform data queried using the
              CURVe? query. Data values are returned as signed decimal integers for analog channel
              and RF time domain data, 4-byte floating point values for RF frequency domain data, or
              hexadecimal values for Digital Collection data with 4 or 8 bytes per point. The
              maximum number of ASCII data points that can be queried using the CURVe? query is 1
              million points. If more than 1 million points are desired, you must use one of the
              binary encodings. If ASCII is the value, then ``BN_Fmt`` and ``BYT_Or`` are ignored.
            - ``FAStest`` specifies the encoding which results in the fastest waveform data transfer
              rate. This sets the following: ``WFMOutpre:ENCdg BINary``, ``WFMOutpre:BIN_Fmt RI``
              and ``WFMOutpre:BYT_Or MSB``.
            - ``RIBinary`` specifies the signed integer data point format, with the most significant
              byte transferred first.
            - ``DATa:WIDTH`` is set to 1, the range is from -128 through 127. When ``DATa:WIDTH`` is
              2, the range is from -32,768 through 32,768. Center screen is 0 (zero). The upper
              limit is the top of the screen and the lower limit is the bottom of the screen. The
              default encoding is RIBINARY. This sets the following: ``WFMOutpre:ENCdg BINary``,
              ``WFMOutpre:BN_Fmt RI`` and ``WFMOutpre:BYT_Or MSB``.
            - ``RPBinary`` specifies the positive integer data-point representation, with the most
              significant byte transferred first.
            - ``BYT_Nr`` is 1, the range of data values is 0 through 255. When ``BYT_Nr`` is 2, the
              range of data values is 0 to 65,535. The center of the screen is 127 for 1-byte data
              and is 32768 for 2-byte data. The upper limit is the top of the screen and the lower
              limit is the bottom of the screen. This sets the following:
              ``:WFMOutpre:ENCdg BINary``,``:WFMOutpre:BN_Fmt RP`` and ``WFMOutpre:BYT_Or MSB``.
            - ``SRIbinary`` specifies the signed integer format. It is the same as RIBinary except
              that the byte order is swapped, meaning that the least significant byte is transferred
              first. This sets the following: ``WFMOutpre:ENCdg BINary``, ``WFMOutpre:BIN_Fmt RI``
              and ``WFMOutpre:BYT_Or LSB``.
            - ``SRPbinary`` specifies the positive integer format. It is the same as RPBinary except
              that the byte order is swapped, meaning that the least significant byte is transferred
              first. This sets the following: ``WFMOutpre:ENCdg BINary``, ``WFMOutpre:BN_Fmt RP``
              and ``WFMOutpre:BYT_Or LSB``.
            - ``FPbinary`` specifies floating point binary. It automatically forces ``DATa:WIDTH``
              to 4 and ``BYT_OR`` to MSB (most significant byte transmitted first).
            - ``SFPbinary`` specifies floating point binary. It automatically forces ``DATa:WIDTH``
              to 4 and ``BYT_OR`` to LSB (least significant byte transmitted first).
        """
        return self._encdg

    @property
    def source(self) -> DataSource:
        """Return the ``DATa:SOUrce`` command.

        Description:
            - This command specifies the source waveform to be transferred from the oscilloscope
              using the CURVe? query. The valid waveform sources are CH1-CH4, MATH, REF1-REF4,
              D0-D15, DIGital, ``RF_AMPlitude``, ``RF_FREQuency``, ``RF_PHASe``, ``RF_NORMal``,
              ``RF_AVErage``, ``RF_MAXHold``, and ``RF_MINHold``. Setting ``DATa:SOUrce``
              automatically constrains the following to valid values for the specified source
              waveform: ``WFMOutpre:BYT_Nr``, ``WFMOutpre:BIT_Nr`` and ``WFMOutpre:BN_Fmt``.

        Usage:
            - Using the ``.query()`` method will send the ``DATa:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:SOUrce?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:SOUrce value`` command.

        SCPI Syntax:
            ```
            - DATa:SOUrce {CH<x>|MATH|REF<x>|D<x>|DIGital|RF_AMPlitude|RF_FREQuency|RF_PHASe|RF_NORMal|RF_AVErage|RF_MAXHold|RF_MINHold}
            - DATa:SOUrce?
            ```

        Info:
            - ``CH<x>`` specifies which analog channel waveform data will be transferred from the
              oscilloscope to the controller, channels 1 through 4.
            - ``MATH`` specifies that the Math waveform data will be transferred from the
              oscilloscope to the controller.
            - ``REF<x>`` specifies which Reference waveform data will be transferred from the
              oscilloscope to the controller.
            - ``D<x>`` specifies which digital channel waveform data will be transferred from the
              oscilloscope to the controller. (Requires installation of option 3-MSO.).
            - ``DIGital`` specifies that the Digital Collection waveform data will be transferred
              from the oscilloscope to the controller. (Requires installation of option 3-MSO.).
            - ``RF_NORMal`` specify that the RF data will be transferred from the oscilloscope to
              the controller.
            - ``RF_AVErage`` specify that the RF data will be transferred from the oscilloscope to
              the controller.
            - ``RF_MAXHold`` specify that the RF data will be transferred from the oscilloscope to
              the controller.
            - ``RF_MINHold`` specify that the RF data will be transferred from the oscilloscope to
              the controller.
        """  # noqa: E501
        return self._source

    @property
    def start(self) -> DataStart:
        """Return the ``DATa:STARt`` command.

        Description:
            - This command sets or queries the starting data point for waveform transfer. This
              command allows for the transfer of partial waveforms to and from the instrument.

        Usage:
            - Using the ``.query()`` method will send the ``DATa:STARt?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:STARt?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:STARt value`` command.

        SCPI Syntax:
            ```
            - DATa:STARt <NR1>
            - DATa:STARt?
            ```

        Info:
            - ``<NR1>`` is the first data point that will be transferred, which ranges from 1 to the
              record length. Data will be transferred from <NR1> to ``DATa:STOP`` or the record
              length, whichever is less. If <NR1> is greater than the record length, the last data
              point in the record is transferred.
            - ``DATa:STARt`` and ``DATa:STOP`` are order independent. When ``DATa:STOP`` is greater
              than ``DATa:STARt``, the values will be swapped internally for the CURVE? query.
        """
        return self._start

    @property
    def stop(self) -> DataStop:
        """Return the ``DATa:STOP`` command.

        Description:
            - This command sets or queries the last data point that will be transferred when using
              the CURVE? query. This command allows for the transfer of partial waveforms to the
              controller. Changes to the record length value are not automatically reflected in the
              ``data:stop`` value. As record length is varied, the ``DATa:STOP`` value must be
              explicitly changed to ensure the entire record is transmitted. In other words, curve
              results will not automatically and correctly reflect increases in record length if the
              distance from ``DATa:STARt`` to ``DATa:STOP`` stays smaller than the increased record
              length.

        Usage:
            - Using the ``.query()`` method will send the ``DATa:STOP?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:STOP?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:STOP value`` command.

        SCPI Syntax:
            ```
            - DATa:STOP <NR1>
            - DATa:STOP?
            ```

        Info:
            - ``<NR1>`` is the last data point that will be transferred, which ranges from 1 to the
              record length. If <NR1> is greater than the record length, then data will be
              transferred up to the record length. If both ``DATa:STARt`` and ``DATa:STOP`` are
              greater than the record length, the last data point in the record is returned.
        """
        return self._stop

    @property
    def width(self) -> DataWidth:
        """Return the ``DATa:WIDth`` command.

        Description:
            - This command specifies the width, in bytes per point, for waveform data transferred
              from the instrument via the CURVe? query. (This command is synonymous with
              ``WFMOutpre:BYT_Nr``.)

        Usage:
            - Using the ``.query()`` method will send the ``DATa:WIDth?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:WIDth?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:WIDth value`` command.

        SCPI Syntax:
            ```
            - DATa:WIDth <NR1>
            - DATa:WIDth?
            ```

        Info:
            - ``<NR1>`` is an integer that indicates the number of bytes per point for the outgoing
              waveform data when queried using the CURVe? command. For analog channels, NR1 can be 1
              or 2.
        """
        return self._width



# pyright: reportConstantRedefinition=none
"""The localnode commands module.

These commands are used in the following models:
DMM7510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - localnode.access
    - localnode.gettime()
    - localnode.internaltemp
    - localnode.linefreq
    - localnode.model
    - localnode.password
    - localnode.prompts
    - localnode.prompts4882
    - localnode.serialno
    - localnode.settime()
    - localnode.showevents
    - localnode.version
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Localnode(BaseTSPCmd):
    """The ``localnode`` command tree.

    Constants:
        - ``.ACCESS_EXCLUSIVE``: Allows access by one remote interface at a time with logins
          required from other interfaces.
        - ``.ACCESS_FULL``: Full access for all users from all interfaces.
        - ``.ACCESS_LOCKOUT``: Allows access by one interface (including the front panel) at a time
          with passwords required on all interfaces.
        - ``.ACCESS_PROTECTED``: Allows access by one remote interface at a time with passwords
          required on all interfaces.
        - ``.DISABLE``: Do not generate prompts in response to command messages.
        - ``.ENABLE``: Generate prompts in response to command messages.

    Properties and methods:
        - ``.access``: The ``localnode.access`` attribute.
        - ``.gettime()``: The ``localnode.gettime()`` function.
        - ``.internaltemp``: The ``localnode.internaltemp`` attribute.
        - ``.linefreq``: The ``localnode.linefreq`` attribute.
        - ``.model``: The ``localnode.model`` attribute.
        - ``.password``: The ``localnode.password`` attribute.
        - ``.prompts``: The ``localnode.prompts`` attribute.
        - ``.prompts4882``: The ``localnode.prompts4882`` attribute.
        - ``.serialno``: The ``localnode.serialno`` attribute.
        - ``.settime()``: The ``localnode.settime()`` function.
        - ``.showevents``: The ``localnode.showevents`` attribute.
        - ``.version``: The ``localnode.version`` attribute.
    """

    ACCESS_EXCLUSIVE = "localnode.ACCESS_EXCLUSIVE"
    """str: Allows access by one remote interface at a time with logins required from other interfaces."""  # noqa: E501
    ACCESS_FULL = "localnode.ACCESS_FULL"
    """str: Full access for all users from all interfaces."""
    ACCESS_LOCKOUT = "localnode.ACCESS_LOCKOUT"
    """str: Allows access by one interface (including the front panel) at a time with passwords required on all interfaces."""  # noqa: E501
    ACCESS_PROTECTED = "localnode.ACCESS_PROTECTED"
    """str: Allows access by one remote interface at a time with passwords required on all interfaces."""  # noqa: E501
    DISABLE = "localnode.DISABLE"
    """str: Do not generate prompts in response to command messages."""
    ENABLE = "localnode.ENABLE"
    """str: Generate prompts in response to command messages."""

    def __init__(
        self, device: Optional["TSPControl"] = None, cmd_syntax: str = "localnode"
    ) -> None:
        super().__init__(device, cmd_syntax)

    @property
    def access(self) -> str:
        """Access the ``localnode.access`` attribute.

        Description:
            - This attribute contains the type of access users have to the instrument through
              different interfaces.

        Usage:
            - Accessing this property will send the ``print(localnode.access)`` query.
            - Setting this property to a value will send the ``localnode.access = value`` command.

        TSP Syntax:
            ```
            - localnode.access = value
            - print(localnode.access)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".access"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.access)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.access`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @access.setter
    def access(self, value: Union[str, float]) -> None:
        """Access the ``localnode.access`` attribute.

        Description:
            - This attribute contains the type of access users have to the instrument through
              different interfaces.

        Usage:
            - Accessing this property will send the ``print(localnode.access)`` query.
            - Setting this property to a value will send the ``localnode.access = value`` command.

        TSP Syntax:
            ```
            - localnode.access = value
            - print(localnode.access)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".access", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.access = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.access`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def internaltemp(self) -> str:
        """Access the ``localnode.internaltemp`` attribute.

        Description:
            - This attribute returns the internal temperature of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.internaltemp)`` query.

        TSP Syntax:
            ```
            - print(localnode.internaltemp)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".internaltemp"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.internaltemp)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.internaltemp`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def linefreq(self) -> str:
        """Access the ``localnode.linefreq`` attribute.

        Description:
            - This attribute contains the power line frequency setting that is used for NPLC
              calculations.

        Usage:
            - Accessing this property will send the ``print(localnode.linefreq)`` query.

        TSP Syntax:
            ```
            - print(localnode.linefreq)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".linefreq"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.linefreq)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.linefreq`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def model(self) -> str:
        """Access the ``localnode.model`` attribute.

        Description:
            - This attribute stores the model number.

        Usage:
            - Accessing this property will send the ``print(localnode.model)`` query.

        TSP Syntax:
            ```
            - print(localnode.model)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".model"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.model)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.model`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def password(self) -> str:
        """Access the ``localnode.password`` attribute.

        Description:
            - This attribute stores the instrument password.

        Usage:
            - Setting this property to a value will send the ``localnode.password = value`` command.

        TSP Syntax:
            ```
            - localnode.password = value
            ```

        Raises:
            AttributeError: Indicates that this attribute is write-only.
        """
        if self._device.command_syntax_enabled:  # type: ignore[union-attr]
            return self._cmd_syntax + ".password"
        msg = f"``{self._cmd_syntax}.password`` is a write-only attribute."
        raise AttributeError(msg)

    @password.setter
    def password(self, value: Union[str, float]) -> None:
        """Access the ``localnode.password`` attribute.

        Description:
            - This attribute stores the instrument password.

        Usage:
            - Setting this property to a value will send the ``localnode.password = value`` command.

        TSP Syntax:
            ```
            - localnode.password = value
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.password = {value}"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.password`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def prompts(self) -> str:
        """Access the ``localnode.prompts`` attribute.

        Description:
            - This attribute determines if the instrument generates prompts in response to command
              messages.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts)`` query.
            - Setting this property to a value will send the ``localnode.prompts = value`` command.

        TSP Syntax:
            ```
            - localnode.prompts = value
            - print(localnode.prompts)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".prompts"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.prompts)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @prompts.setter
    def prompts(self, value: Union[str, float]) -> None:
        """Access the ``localnode.prompts`` attribute.

        Description:
            - This attribute determines if the instrument generates prompts in response to command
              messages.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts)`` query.
            - Setting this property to a value will send the ``localnode.prompts = value`` command.

        TSP Syntax:
            ```
            - localnode.prompts = value
            - print(localnode.prompts)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".prompts", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.prompts = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def prompts4882(self) -> str:
        """Access the ``localnode.prompts4882`` attribute.

        Description:
            - This attribute enables and disables the generation of prompts for IEEE Std 488.2
              common commands.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts4882)`` query.
            - Setting this property to a value will send the ``localnode.prompts4882 = value``
              command.

        TSP Syntax:
            ```
            - localnode.prompts4882 = value
            - print(localnode.prompts4882)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".prompts4882"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.prompts4882)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts4882`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @prompts4882.setter
    def prompts4882(self, value: Union[str, float]) -> None:
        """Access the ``localnode.prompts4882`` attribute.

        Description:
            - This attribute enables and disables the generation of prompts for IEEE Std 488.2
              common commands.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts4882)`` query.
            - Setting this property to a value will send the ``localnode.prompts4882 = value``
              command.

        TSP Syntax:
            ```
            - localnode.prompts4882 = value
            - print(localnode.prompts4882)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".prompts4882", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.prompts4882 = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts4882`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def serialno(self) -> str:
        """Access the ``localnode.serialno`` attribute.

        Description:
            - This attribute stores the serial number of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.serialno)`` query.

        TSP Syntax:
            ```
            - print(localnode.serialno)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".serialno"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.serialno)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.serialno`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def showevents(self) -> str:
        """Access the ``localnode.showevents`` attribute.

        Description:
            - This attribute sets whether or not the instrument automatically outputs generated
              events to the remote interface.

        Usage:
            - Accessing this property will send the ``print(localnode.showevents)`` query.
            - Setting this property to a value will send the ``localnode.showevents = value``
              command.

        TSP Syntax:
            ```
            - localnode.showevents = value
            - print(localnode.showevents)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".showevents"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.showevents)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.showevents`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @showevents.setter
    def showevents(self, value: Union[str, float]) -> None:
        """Access the ``localnode.showevents`` attribute.

        Description:
            - This attribute sets whether or not the instrument automatically outputs generated
              events to the remote interface.

        Usage:
            - Accessing this property will send the ``print(localnode.showevents)`` query.
            - Setting this property to a value will send the ``localnode.showevents = value``
              command.

        TSP Syntax:
            ```
            - localnode.showevents = value
            - print(localnode.showevents)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".showevents", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.showevents = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.showevents`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def version(self) -> str:
        """Access the ``localnode.version`` attribute.

        Description:
            - This attribute stores the firmware version of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.version)`` query.

        TSP Syntax:
            ```
            - print(localnode.version)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".version"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.version)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.version`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def gettime(self) -> None:
        """Run the ``localnode.gettime()`` function.

        Description:
            - This function retrieves the instrument date and time.

        TSP Syntax:
            ```
            - localnode.gettime()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.gettime()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.gettime()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def settime(
        self,
        year: Optional[str] = None,
        month: Optional[int] = None,
        day: Optional[int] = None,
        hour: Optional[int] = None,
        minute: Optional[int] = None,
        second: Optional[int] = None,
    ) -> None:
        """Run the ``localnode.settime()`` function.

        Description:
            - This function sets the date and time of the instrument.

        TSP Syntax:
            ```
            - localnode.settime()
            ```

        Args:
            year (optional): Year; must be more than 1970.
            month (optional): Month (1 to 12).
            day (optional): Day (1 to 31).
            hour (optional): Hour in 24-hour time format (0 to 23).
            minute (optional): Minute (0 to 59).
            second (optional): Second (0 to 59).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    year,
                    month,
                    day,
                    hour,
                    minute,
                    second,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.settime({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.settime()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The sv commands module.

These commands are used in the following models:
DPO7AX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SV:WINDOW {KAISerbessel|RECTangular|HAMMing|HANNing|BLACkmanharris|FLATtop2}
    - SV:WINDOW?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SvWindow(SCPICmdWrite, SCPICmdRead):
    """The ``SV:WINDOW`` command.

    Description:
        - This command sets or queries the window type used by the windowing function of the
          Spectrum View. The windowing function is a Fast Fourier Transform (FFT) technique used to
          minimize the discontinuities between successive frames of an RF time domain signal. The
          default window type is Blackman-Harris.

    Usage:
        - Using the ``.query()`` method will send the ``SV:WINDOW?`` query.
        - Using the ``.verify(value)`` method will send the ``SV:WINDOW?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SV:WINDOW value`` command.

    SCPI Syntax:
        ```
        - SV:WINDOW {KAISerbessel|RECTangular|HAMMing|HANNing|BLACkmanharris|FLATtop2}
        - SV:WINDOW?
        ```

    Info:
        - ``KAISerbessel`` specifies the Kaiser-Bessel window type (a high or moderate resolution
          window).
        - ``RECTangular`` specifies the Rectangular window type (a window function is equivalent to
          multiplying all gate data by one).
        - ``HAMMing`` specifies the Hamming window type (a high or moderate resolution window based
          on a cosine series).
        - ``HANNing`` specifies the Hanning window type (a high or moderate resolution window based
          on a cosine series).
        - ``BLACkmanharris`` specifies the Blackman-Harris window type (a low-resolution (high
          dynamic range) window based on a cosine series).
        - ``FLATtop2`` specifies the Flattop2 window type (a low-resolution (high dynamic range)
          window).
    """


class Sv(SCPICmdRead):
    """The ``SV`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SV?`` query.
        - Using the ``.verify(value)`` method will send the ``SV?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.window``: The ``SV:WINDOW`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SV") -> None:
        super().__init__(device, cmd_syntax)
        self._window = SvWindow(device, f"{self._cmd_syntax}:WINDOW")

    @property
    def window(self) -> SvWindow:
        """Return the ``SV:WINDOW`` command.

        Description:
            - This command sets or queries the window type used by the windowing function of the
              Spectrum View. The windowing function is a Fast Fourier Transform (FFT) technique used
              to minimize the discontinuities between successive frames of an RF time domain signal.
              The default window type is Blackman-Harris.

        Usage:
            - Using the ``.query()`` method will send the ``SV:WINDOW?`` query.
            - Using the ``.verify(value)`` method will send the ``SV:WINDOW?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SV:WINDOW value`` command.

        SCPI Syntax:
            ```
            - SV:WINDOW {KAISerbessel|RECTangular|HAMMing|HANNing|BLACkmanharris|FLATtop2}
            - SV:WINDOW?
            ```

        Info:
            - ``KAISerbessel`` specifies the Kaiser-Bessel window type (a high or moderate
              resolution window).
            - ``RECTangular`` specifies the Rectangular window type (a window function is equivalent
              to multiplying all gate data by one).
            - ``HAMMing`` specifies the Hamming window type (a high or moderate resolution window
              based on a cosine series).
            - ``HANNing`` specifies the Hanning window type (a high or moderate resolution window
              based on a cosine series).
            - ``BLACkmanharris`` specifies the Blackman-Harris window type (a low-resolution (high
              dynamic range) window based on a cosine series).
            - ``FLATtop2`` specifies the Flattop2 window type (a low-resolution (high dynamic range)
              window).
        """
        return self._window



"""The filesystem commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - FILESystem:COPy {<source QString>,<destination QString>}
    - FILESystem:CWD {<new working directory path>}
    - FILESystem:DELEte <file path>
    - FILESystem:DIR?
    - FILESystem:FORMat
    - FILESystem:FREESpace?
    - FILESystem:LDIR?
    - FILESystem:MKDir <directory path>
    - FILESystem:MOUNT:AVAILable?
    - FILESystem:MOUNT:DRIve <Qstring>
    - FILESystem:MOUNT:LIST?
    - FILESystem:READFile <QString>
    - FILESystem:REName <old file path>,<new file path>
    - FILESystem:RMDir <directory path>
    - FILESystem:UNMOUNT:DRIve QString
    - FILESystem:WRITEFile <file path>, <data>
    - FILESystem?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class FilesystemWritefile(SCPICmdWrite):
    """The ``FILESystem:WRITEFile`` command.

    Description:
        - Writes the specified block data to a file in the oscilloscope current working directory.
          If the specified file does not exist or is not readable, an appropriate error event is
          posted.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:WRITEFile value`` command.

    SCPI Syntax:
        ```
        - FILESystem:WRITEFile <file path>, <data>
        ```

    Info:
        - ``<file path>`` is the quoted string that defines the file name and path. If the path is
          within the current working directory, specify the file name only.
        - ``<data>`` can be either DEFINITE LENGTH encoding or INDEFINITE LENGTH ARBITRARY BLOCK
          PROGRAM DATA encoding as described in IEEE488.2.
    """


class FilesystemUnmountDrive(SCPICmdWrite):
    """The ``FILESystem:UNMOUNT:DRIve`` command.

    Description:
        - This command attempts to un-mount the network drive specified by the quoted string
          argument.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:UNMOUNT:DRIve value``
          command.

    SCPI Syntax:
        ```
        - FILESystem:UNMOUNT:DRIve QString
        ```
    """


class FilesystemUnmount(SCPICmdRead):
    """The ``FILESystem:UNMOUNT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:UNMOUNT?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:UNMOUNT?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.drive``: The ``FILESystem:UNMOUNT:DRIve`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._drive = FilesystemUnmountDrive(device, f"{self._cmd_syntax}:DRIve")

    @property
    def drive(self) -> FilesystemUnmountDrive:
        """Return the ``FILESystem:UNMOUNT:DRIve`` command.

        Description:
            - This command attempts to un-mount the network drive specified by the quoted string
              argument.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:UNMOUNT:DRIve value``
              command.

        SCPI Syntax:
            ```
            - FILESystem:UNMOUNT:DRIve QString
            ```
        """
        return self._drive


class FilesystemRmdir(SCPICmdWrite):
    """The ``FILESystem:RMDir`` command.

    Description:
        - Deletes a named directory. This command deletes the specified directory and all of its
          contents. The directory must not be a read-only directory.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:RMDir value`` command.

    SCPI Syntax:
        ```
        - FILESystem:RMDir <directory path>
        ```

    Info:
        - ``<directory path>`` is a quoted string that defines the directory name and path. If the
          file path is within the current working directory, you need only specify the file name.
    """


class FilesystemRename(SCPICmdWrite):
    """The ``FILESystem:REName`` command.

    Description:
        - Assigns a new name to an existing file.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:REName value`` command.

    SCPI Syntax:
        ```
        - FILESystem:REName <old file path>,<new file path>
        ```

    Info:
        - ``<old file path>`` is a quoted string that defines the file name and path. If the file
          path is within the current working directory, you need only specify the file name.
        - ``<new file path>`` is a quoted string that defines the file name and path. If the file
          path is within the current working directory, you need only specify the file name.
    """


class FilesystemReadfile(SCPICmdWrite):
    """The ``FILESystem:READFile`` command.

    Description:
        - This command writes the contents of the specified file to the current interface. If the
          specified file does not exist or is not readable, an appropriate error event is posted.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:READFile value`` command.

    SCPI Syntax:
        ```
        - FILESystem:READFile <QString>
        ```

    Info:
        - ``<QString>`` is a quoted string that defines the file name and path. If the file path is
          within the current working directory, you need only specify the file name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class FilesystemMountList(SCPICmdRead):
    """The ``FILESystem:MOUNT:LIST`` command.

    Description:
        - This query returns a comma-separated list of the mounted network drives, including the
          drive letter, server identity (DNS name or IP address), mount path and type. If no network
          drives are mounted, an empty string is returned. Mount types are either NFS or CIFS (for
          Microsoft Windows networks).

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:MOUNT:LIST?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT:LIST?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - FILESystem:MOUNT:LIST?
        ```
    """


class FilesystemMountDrive(SCPICmdWrite):
    """The ``FILESystem:MOUNT:DRIve`` command.

    Description:
        - This command attempts to mount the network drive specified by the quoted string argument.
          The query form takes a quoted string argument specifying the drive letter, and returns a
          Boolean to indicate whether the specified drive letter is mounted. 1 = mounted; 0 = not
          mounted. You can get the details of the mounted drives by querying
          ``FILESYSTEM:MOUNT:LIST``

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:MOUNT:DRIve value`` command.

    SCPI Syntax:
        ```
        - FILESystem:MOUNT:DRIve <Qstring>
        ```

    Info:
        - ``Drive Name:`` The drive name to use, which should be a case insensitive single letter
          followed by a colon. The drive name must be a letter between 'I' and 'Z', inclusive.
          Drives A: through D: are not used and drives E: through H: are reserved for the USB ports.
        - ``Server Identity:`` One of.
        - ``Path:`` The path to be mounted; e.g. /this/that/mydir.
        - ``Domain:`` The domain/workgroup of the target mount.
        - ``User Name:`` The user name.
        - ``User Password:`` The user password.
    """


class FilesystemMountAvailable(SCPICmdRead):
    """The ``FILESystem:MOUNT:AVAILable`` command.

    Description:
        - This query returns a comma-separated list of available drive letters that can be used for
          mounting network drives.

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:MOUNT:AVAILable?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT:AVAILable?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - FILESystem:MOUNT:AVAILable?
        ```
    """


class FilesystemMount(SCPICmdRead):
    """The ``FILESystem:MOUNT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:MOUNT?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.available``: The ``FILESystem:MOUNT:AVAILable`` command.
        - ``.drive``: The ``FILESystem:MOUNT:DRIve`` command.
        - ``.list``: The ``FILESystem:MOUNT:LIST`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._available = FilesystemMountAvailable(device, f"{self._cmd_syntax}:AVAILable")
        self._drive = FilesystemMountDrive(device, f"{self._cmd_syntax}:DRIve")
        self._list = FilesystemMountList(device, f"{self._cmd_syntax}:LIST")

    @property
    def available(self) -> FilesystemMountAvailable:
        """Return the ``FILESystem:MOUNT:AVAILable`` command.

        Description:
            - This query returns a comma-separated list of available drive letters that can be used
              for mounting network drives.

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:MOUNT:AVAILable?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT:AVAILable?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - FILESystem:MOUNT:AVAILable?
            ```
        """
        return self._available

    @property
    def drive(self) -> FilesystemMountDrive:
        """Return the ``FILESystem:MOUNT:DRIve`` command.

        Description:
            - This command attempts to mount the network drive specified by the quoted string
              argument. The query form takes a quoted string argument specifying the drive letter,
              and returns a Boolean to indicate whether the specified drive letter is mounted. 1 =
              mounted; 0 = not mounted. You can get the details of the mounted drives by querying
              ``FILESYSTEM:MOUNT:LIST``

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:MOUNT:DRIve value``
              command.

        SCPI Syntax:
            ```
            - FILESystem:MOUNT:DRIve <Qstring>
            ```

        Info:
            - ``Drive Name:`` The drive name to use, which should be a case insensitive single
              letter followed by a colon. The drive name must be a letter between 'I' and 'Z',
              inclusive. Drives A: through D: are not used and drives E: through H: are reserved for
              the USB ports.
            - ``Server Identity:`` One of.
            - ``Path:`` The path to be mounted; e.g. /this/that/mydir.
            - ``Domain:`` The domain/workgroup of the target mount.
            - ``User Name:`` The user name.
            - ``User Password:`` The user password.
        """
        return self._drive

    @property
    def list(self) -> FilesystemMountList:
        """Return the ``FILESystem:MOUNT:LIST`` command.

        Description:
            - This query returns a comma-separated list of the mounted network drives, including the
              drive letter, server identity (DNS name or IP address), mount path and type. If no
              network drives are mounted, an empty string is returned. Mount types are either NFS or
              CIFS (for Microsoft Windows networks).

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:MOUNT:LIST?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT:LIST?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - FILESystem:MOUNT:LIST?
            ```
        """
        return self._list


class FilesystemMkdir(SCPICmdWrite):
    """The ``FILESystem:MKDir`` command.

    Description:
        - Creates a new folder.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:MKDir value`` command.

    SCPI Syntax:
        ```
        - FILESystem:MKDir <directory path>
        ```

    Info:
        - ``<directory path>`` is a quoted string that specifies the directory to create.
    """


class FilesystemLdir(SCPICmdRead):
    """The ``FILESystem:LDIR`` command.

    Description:
        - Returns a comma separated list of every file, file size, type, modification date and time,
          and directory in the folder referred to by the ``FILESYSTEM:CWD`` command. This is
          different than the ``:DIR`` query in that it provides a long output format with the file
          size, type, and modification date/time. Each entry is a semicolon separated list: < file
          name>;<type>;<size in bytes>;<date>;<time>

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:LDIR?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:LDIR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - FILESystem:LDIR?
        ```
    """


class FilesystemFreespace(SCPICmdRead):
    """The ``FILESystem:FREESpace`` command.

    Description:
        - Returns the number of bytes of free space on the current drive.

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:FREESpace?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:FREESpace?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - FILESystem:FREESpace?
        ```
    """


class FilesystemFormat(SCPICmdWriteNoArguments):
    """The ``FILESystem:FORMat`` command.

    Description:
        - Formats a mass storage device. This command should be used with extreme caution as it
          causes all data on the specified mass storage device to be lost. Drive letters (e.g., E:)
          are case sensitive and must be upper case. For all other FILESYSTEM commands, drives
          letters are not case sensitive. Example: ``FILES:FORMAT`` 'E:/' Formats the USB flash
          drive installed in the oscilloscope's front panel USB port.

    Usage:
        - Using the ``.write()`` method will send the ``FILESystem:FORMat`` command.

    SCPI Syntax:
        ```
        - FILESystem:FORMat
        ```
    """


class FilesystemDir(SCPICmdRead):
    """The ``FILESystem:DIR`` command.

    Description:
        - This query-only command returns a comma separated list of quoted strings. Each string
          contains the name of a file or directory in the folder referred to by the
          ``FILESYSTEM:CWD`` command.

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem:DIR?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:DIR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - FILESystem:DIR?
        ```
    """


class FilesystemDelete(SCPICmdWrite):
    r"""The ``FILESystem:DELEte`` command.

    Description:
        - This command deletes a named file. If you specify a directory name, it will delete the
          directory and all of its contents, the same as the RMDir command. You can also specify the
          filename as \*.* to delete all of the files in the current or specified directory.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:DELEte value`` command.

    SCPI Syntax:
        ```
        - FILESystem:DELEte <file path>
        ```

    Info:
        - ``<file path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
        - ``*.*`` will delete all files and subdirectories within the current working directory.
    """


class FilesystemCwd(SCPICmdWrite):
    """The ``FILESystem:CWD`` command.

    Description:
        - This command specifies the current working directory (CWD) for FILESystem commands. The
          default working directory is 'E:/'. Anytime you use this command to change the directory,
          the directory that you specify is retained as the current working directory until you
          either change the directory or you delete the directory. If you delete the current working
          directory, the oscilloscope resets current working directory to the default directory the
          next time the oscilloscope is powered on or the next time you execute a file system
          command. This command supports the permutations of file and directory names supported by
          Microsoft Windows: Relative path names; for example, './Temp' Absolute path names; for
          example, 'E:/MyWaveform' Implied relative path names; for example 'newfile.txt' becomes
          'E:/TekScope/newfile.txt' if the current working directory is 'E:/TekScope'

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:CWD value`` command.

    SCPI Syntax:
        ```
        - FILESystem:CWD {<new working directory path>}
        ```

    Info:
        - ``<new working directory path>`` is a quoted string that defines the current working; a
          directory name can be up to 128 characters.
    """


class FilesystemCopy(SCPICmdWrite):
    """The ``FILESystem:COPy`` command.

    Description:
        - This command copies a named file to a new file. The new file may be in a totally separate
          directory than the old file. You can only copy one file at a time using this command. Wild
          card characters are not allowed.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESystem:COPy value`` command.

    SCPI Syntax:
        ```
        - FILESystem:COPy {<source QString>,<destination QString>}
        ```

    Info:
        - ``QString`` is a quoted string that defines the file name and path. If the file path is
          within the current working directory, you need only specify the file name.
    """


#  pylint: disable=too-many-instance-attributes
class Filesystem(SCPICmdRead):
    """The ``FILESystem`` command.

    Description:
        - This query-only command returns the directory listing of the current working directory.
          This query is the same as the ``FILESystem:DIR?`` query.

    Usage:
        - Using the ``.query()`` method will send the ``FILESystem?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - FILESystem?
        ```

    Properties:
        - ``.copy``: The ``FILESystem:COPy`` command.
        - ``.cwd``: The ``FILESystem:CWD`` command.
        - ``.delete``: The ``FILESystem:DELEte`` command.
        - ``.dir``: The ``FILESystem:DIR`` command.
        - ``.format``: The ``FILESystem:FORMat`` command.
        - ``.freespace``: The ``FILESystem:FREESpace`` command.
        - ``.ldir``: The ``FILESystem:LDIR`` command.
        - ``.mkdir``: The ``FILESystem:MKDir`` command.
        - ``.mount``: The ``FILESystem:MOUNT`` command tree.
        - ``.readfile``: The ``FILESystem:READFile`` command.
        - ``.rename``: The ``FILESystem:REName`` command.
        - ``.rmdir``: The ``FILESystem:RMDir`` command.
        - ``.unmount``: The ``FILESystem:UNMOUNT`` command tree.
        - ``.writefile``: The ``FILESystem:WRITEFile`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "FILESystem"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._copy = FilesystemCopy(device, f"{self._cmd_syntax}:COPy")
        self._cwd = FilesystemCwd(device, f"{self._cmd_syntax}:CWD")
        self._delete = FilesystemDelete(device, f"{self._cmd_syntax}:DELEte")
        self._dir = FilesystemDir(device, f"{self._cmd_syntax}:DIR")
        self._format = FilesystemFormat(device, f"{self._cmd_syntax}:FORMat")
        self._freespace = FilesystemFreespace(device, f"{self._cmd_syntax}:FREESpace")
        self._ldir = FilesystemLdir(device, f"{self._cmd_syntax}:LDIR")
        self._mkdir = FilesystemMkdir(device, f"{self._cmd_syntax}:MKDir")
        self._mount = FilesystemMount(device, f"{self._cmd_syntax}:MOUNT")
        self._readfile = FilesystemReadfile(device, f"{self._cmd_syntax}:READFile")
        self._rename = FilesystemRename(device, f"{self._cmd_syntax}:REName")
        self._rmdir = FilesystemRmdir(device, f"{self._cmd_syntax}:RMDir")
        self._unmount = FilesystemUnmount(device, f"{self._cmd_syntax}:UNMOUNT")
        self._writefile = FilesystemWritefile(device, f"{self._cmd_syntax}:WRITEFile")

    @property
    def copy(self) -> FilesystemCopy:
        """Return the ``FILESystem:COPy`` command.

        Description:
            - This command copies a named file to a new file. The new file may be in a totally
              separate directory than the old file. You can only copy one file at a time using this
              command. Wild card characters are not allowed.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:COPy value`` command.

        SCPI Syntax:
            ```
            - FILESystem:COPy {<source QString>,<destination QString>}
            ```

        Info:
            - ``QString`` is a quoted string that defines the file name and path. If the file path
              is within the current working directory, you need only specify the file name.
        """
        return self._copy

    @property
    def cwd(self) -> FilesystemCwd:
        """Return the ``FILESystem:CWD`` command.

        Description:
            - This command specifies the current working directory (CWD) for FILESystem commands.
              The default working directory is 'E:/'. Anytime you use this command to change the
              directory, the directory that you specify is retained as the current working directory
              until you either change the directory or you delete the directory. If you delete the
              current working directory, the oscilloscope resets current working directory to the
              default directory the next time the oscilloscope is powered on or the next time you
              execute a file system command. This command supports the permutations of file and
              directory names supported by Microsoft Windows: Relative path names; for example,
              './Temp' Absolute path names; for example, 'E:/MyWaveform' Implied relative path
              names; for example 'newfile.txt' becomes 'E:/TekScope/newfile.txt' if the current
              working directory is 'E:/TekScope'

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:CWD value`` command.

        SCPI Syntax:
            ```
            - FILESystem:CWD {<new working directory path>}
            ```

        Info:
            - ``<new working directory path>`` is a quoted string that defines the current working;
              a directory name can be up to 128 characters.
        """
        return self._cwd

    @property
    def delete(self) -> FilesystemDelete:
        r"""Return the ``FILESystem:DELEte`` command.

        Description:
            - This command deletes a named file. If you specify a directory name, it will delete the
              directory and all of its contents, the same as the RMDir command. You can also specify
              the filename as \*.* to delete all of the files in the current or specified directory.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:DELEte value`` command.

        SCPI Syntax:
            ```
            - FILESystem:DELEte <file path>
            ```

        Info:
            - ``<file path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
            - ``*.*`` will delete all files and subdirectories within the current working directory.
        """
        return self._delete

    @property
    def dir(self) -> FilesystemDir:
        """Return the ``FILESystem:DIR`` command.

        Description:
            - This query-only command returns a comma separated list of quoted strings. Each string
              contains the name of a file or directory in the folder referred to by the
              ``FILESYSTEM:CWD`` command.

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:DIR?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:DIR?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - FILESystem:DIR?
            ```
        """
        return self._dir

    @property
    def format(self) -> FilesystemFormat:
        """Return the ``FILESystem:FORMat`` command.

        Description:
            - Formats a mass storage device. This command should be used with extreme caution as it
              causes all data on the specified mass storage device to be lost. Drive letters (e.g.,
              E:) are case sensitive and must be upper case. For all other FILESYSTEM commands,
              drives letters are not case sensitive. Example: ``FILES:FORMAT`` 'E:/' Formats the USB
              flash drive installed in the oscilloscope's front panel USB port.

        Usage:
            - Using the ``.write()`` method will send the ``FILESystem:FORMat`` command.

        SCPI Syntax:
            ```
            - FILESystem:FORMat
            ```
        """
        return self._format

    @property
    def freespace(self) -> FilesystemFreespace:
        """Return the ``FILESystem:FREESpace`` command.

        Description:
            - Returns the number of bytes of free space on the current drive.

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:FREESpace?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:FREESpace?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - FILESystem:FREESpace?
            ```
        """
        return self._freespace

    @property
    def ldir(self) -> FilesystemLdir:
        """Return the ``FILESystem:LDIR`` command.

        Description:
            - Returns a comma separated list of every file, file size, type, modification date and
              time, and directory in the folder referred to by the ``FILESYSTEM:CWD`` command. This
              is different than the ``:DIR`` query in that it provides a long output format with the
              file size, type, and modification date/time. Each entry is a semicolon separated list:
              < file name>;<type>;<size in bytes>;<date>;<time>

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:LDIR?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:LDIR?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - FILESystem:LDIR?
            ```
        """
        return self._ldir

    @property
    def mkdir(self) -> FilesystemMkdir:
        """Return the ``FILESystem:MKDir`` command.

        Description:
            - Creates a new folder.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:MKDir value`` command.

        SCPI Syntax:
            ```
            - FILESystem:MKDir <directory path>
            ```

        Info:
            - ``<directory path>`` is a quoted string that specifies the directory to create.
        """
        return self._mkdir

    @property
    def mount(self) -> FilesystemMount:
        """Return the ``FILESystem:MOUNT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:MOUNT?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.available``: The ``FILESystem:MOUNT:AVAILable`` command.
            - ``.drive``: The ``FILESystem:MOUNT:DRIve`` command.
            - ``.list``: The ``FILESystem:MOUNT:LIST`` command.
        """
        return self._mount

    @property
    def readfile(self) -> FilesystemReadfile:
        """Return the ``FILESystem:READFile`` command.

        Description:
            - This command writes the contents of the specified file to the current interface. If
              the specified file does not exist or is not readable, an appropriate error event is
              posted.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:READFile value``
              command.

        SCPI Syntax:
            ```
            - FILESystem:READFile <QString>
            ```

        Info:
            - ``<QString>`` is a quoted string that defines the file name and path. If the file path
              is within the current working directory, you need only specify the file name.
        """
        return self._readfile

    @property
    def rename(self) -> FilesystemRename:
        """Return the ``FILESystem:REName`` command.

        Description:
            - Assigns a new name to an existing file.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:REName value`` command.

        SCPI Syntax:
            ```
            - FILESystem:REName <old file path>,<new file path>
            ```

        Info:
            - ``<old file path>`` is a quoted string that defines the file name and path. If the
              file path is within the current working directory, you need only specify the file
              name.
            - ``<new file path>`` is a quoted string that defines the file name and path. If the
              file path is within the current working directory, you need only specify the file
              name.
        """
        return self._rename

    @property
    def rmdir(self) -> FilesystemRmdir:
        """Return the ``FILESystem:RMDir`` command.

        Description:
            - Deletes a named directory. This command deletes the specified directory and all of its
              contents. The directory must not be a read-only directory.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:RMDir value`` command.

        SCPI Syntax:
            ```
            - FILESystem:RMDir <directory path>
            ```

        Info:
            - ``<directory path>`` is a quoted string that defines the directory name and path. If
              the file path is within the current working directory, you need only specify the file
              name.
        """
        return self._rmdir

    @property
    def unmount(self) -> FilesystemUnmount:
        """Return the ``FILESystem:UNMOUNT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``FILESystem:UNMOUNT?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:UNMOUNT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.drive``: The ``FILESystem:UNMOUNT:DRIve`` command.
        """
        return self._unmount

    @property
    def writefile(self) -> FilesystemWritefile:
        """Return the ``FILESystem:WRITEFile`` command.

        Description:
            - Writes the specified block data to a file in the oscilloscope current working
              directory. If the specified file does not exist or is not readable, an appropriate
              error event is posted.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESystem:WRITEFile value``
              command.

        SCPI Syntax:
            ```
            - FILESystem:WRITEFile <file path>, <data>
            ```

        Info:
            - ``<file path>`` is the quoted string that defines the file name and path. If the path
              is within the current working directory, specify the file name only.
            - ``<data>`` can be either DEFINITE LENGTH encoding or INDEFINITE LENGTH ARBITRARY BLOCK
              PROGRAM DATA encoding as described in IEEE488.2.
        """
        return self._writefile



"""The rosc commands module.

These commands are used in the following models:
DPO7AX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - ROSc:SOUrce {INTERnal|EXTernal|TRACking|SAMPle}
    - ROSc:SOUrce?
    - ROSc:STATE?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class RoscState(SCPICmdRead):
    """The ``ROSc:STATE`` command.

    Description:
        - This query-only command returns whether the time base reference oscillator is locked. This
          command will return either LOCKED or UNLOCKED.

    Usage:
        - Using the ``.query()`` method will send the ``ROSc:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ROSc:STATE?
        ```

    Info:
        - ``LOCKED`` indicates the reference oscillator is locked.
        - ``UNLOCKED`` indicates the reference oscillator is not locked.
    """


class RoscSource(SCPICmdWrite, SCPICmdRead):
    """The ``ROSc:SOUrce`` command.

    Description:
        - This command sets or queries the selected source for the time base reference oscillator.
          The reference oscillator locks to this source. Depending on the command argument that you
          specify, you can use an external reference or use the internal crystal oscillator as the
          time base reference. Note: Switching between certain settings is supported at runtime
          (INTERnal and EXTernal). Switching either into or out of other settings (TRACking and
          SAMPle) requires a restart of the instrument. Setting a mode that requires a restart will
          trigger the switch on the next reboot. The query won't reflect the requested time base
          setting until the switch is complete.

    Usage:
        - Using the ``.query()`` method will send the ``ROSc:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ROSc:SOUrce value`` command.

    SCPI Syntax:
        ```
        - ROSc:SOUrce {INTERnal|EXTernal|TRACking|SAMPle}
        - ROSc:SOUrce?
        ```

    Info:
        - ``INTERnal`` specifies the internal 10 MHz crystal oscillator as the time base reference.
        - ``EXTernal`` specifies the user-supplied external signal at 1 ppm as the time base
          reference.
        - ``TRACking`` specifies the user-supplied external signal at 1Kppm as the time base
          reference.
    """


class Rosc(SCPICmdRead):
    """The ``ROSc`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ROSc?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.source``: The ``ROSc:SOUrce`` command.
        - ``.state``: The ``ROSc:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "ROSc") -> None:
        super().__init__(device, cmd_syntax)
        self._source = RoscSource(device, f"{self._cmd_syntax}:SOUrce")
        self._state = RoscState(device, f"{self._cmd_syntax}:STATE")

    @property
    def source(self) -> RoscSource:
        """Return the ``ROSc:SOUrce`` command.

        Description:
            - This command sets or queries the selected source for the time base reference
              oscillator. The reference oscillator locks to this source. Depending on the command
              argument that you specify, you can use an external reference or use the internal
              crystal oscillator as the time base reference. Note: Switching between certain
              settings is supported at runtime (INTERnal and EXTernal). Switching either into or out
              of other settings (TRACking and SAMPle) requires a restart of the instrument. Setting
              a mode that requires a restart will trigger the switch on the next reboot. The query
              won't reflect the requested time base setting until the switch is complete.

        Usage:
            - Using the ``.query()`` method will send the ``ROSc:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:SOUrce?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ROSc:SOUrce value`` command.

        SCPI Syntax:
            ```
            - ROSc:SOUrce {INTERnal|EXTernal|TRACking|SAMPle}
            - ROSc:SOUrce?
            ```

        Info:
            - ``INTERnal`` specifies the internal 10 MHz crystal oscillator as the time base
              reference.
            - ``EXTernal`` specifies the user-supplied external signal at 1 ppm as the time base
              reference.
            - ``TRACking`` specifies the user-supplied external signal at 1Kppm as the time base
              reference.
        """
        return self._source

    @property
    def state(self) -> RoscState:
        """Return the ``ROSc:STATE`` command.

        Description:
            - This query-only command returns whether the time base reference oscillator is locked.
              This command will return either LOCKED or UNLOCKED.

        Usage:
            - Using the ``.query()`` method will send the ``ROSc:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:STATE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ROSc:STATE?
            ```

        Info:
            - ``LOCKED`` indicates the reference oscillator is locked.
            - ``UNLOCKED`` indicates the reference oscillator is not locked.
        """
        return self._state



"""The system commands module.

These commands are used in the following models:
AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SYSTem:DATE <year>,<month>,<day>
    - SYSTem:DATE?
    - SYSTem:ERRor:ALL?
    - SYSTem:ERRor:CODE:ALL?
    - SYSTem:ERRor:CODE:NEXT?
    - SYSTem:ERRor:COUNt?
    - SYSTem:ERRor:DIALog <show_dialog>
    - SYSTem:ERRor:DIALog?
    - SYSTem:ERRor:NEXT?
    - SYSTem:SID:GATeway <address>
    - SYSTem:SID:GATeway?
    - SYSTem:SID:IP <address>
    - SYSTem:SID:IP?
    - SYSTem:SID:MAC?
    - SYSTem:TIME <hour>,<minute>,<second>
    - SYSTem:TIME?
    - SYSTem:VERSion?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SystemVersion(SCPICmdRead):
    """The ``SYSTem:VERSion`` command.

    Description:
        - This query-only command returns the conformed SCPI version of the instrument.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:VERSion?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:VERSion?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:VERSion?
        ```
    """


class SystemTime(SCPICmdWrite, SCPICmdRead):
    """The ``SYSTem:TIME`` command.

    Description:
        - This command sets or returns the system time (hours, minutes and seconds). This command is
          equivalent to the time setting through the Windows Control Panel.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:TIME?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:TIME?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SYSTem:TIME value`` command.

    SCPI Syntax:
        ```
        - SYSTem:TIME <hour>,<minute>,<second>
        - SYSTem:TIME?
        ```
    """


class SystemSidMac(SCPICmdRead):
    """The ``SYSTem:SID:MAC`` command.

    Description:
        - This command returns the MAC address of the Streaming ID port.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:SID:MAC?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:SID:MAC?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:SID:MAC?
        ```
    """


class SystemSidIp(SCPICmdWrite, SCPICmdRead):
    """The ``SYSTem:SID:IP`` command.

    Description:
        - This command sets or returns the IP address of the Streaming ID Ethernet port.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:SID:IP?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:SID:IP?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SYSTem:SID:IP value`` command.

    SCPI Syntax:
        ```
        - SYSTem:SID:IP <address>
        - SYSTem:SID:IP?
        ```
    """


class SystemSidGateway(SCPICmdWrite, SCPICmdRead):
    """The ``SYSTem:SID:GATeway`` command.

    Description:
        - This command sets or returns the gateway address of the Streaming ID Ethernet port.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:SID:GATeway?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:SID:GATeway?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SYSTem:SID:GATeway value`` command.

    SCPI Syntax:
        ```
        - SYSTem:SID:GATeway <address>
        - SYSTem:SID:GATeway?
        ```
    """


class SystemSid(SCPICmdRead):
    """The ``SYSTem:SID`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:SID?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:SID?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.gateway``: The ``SYSTem:SID:GATeway`` command.
        - ``.ip``: The ``SYSTem:SID:IP`` command.
        - ``.mac``: The ``SYSTem:SID:MAC`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._gateway = SystemSidGateway(device, f"{self._cmd_syntax}:GATeway")
        self._ip = SystemSidIp(device, f"{self._cmd_syntax}:IP")
        self._mac = SystemSidMac(device, f"{self._cmd_syntax}:MAC")

    @property
    def gateway(self) -> SystemSidGateway:
        """Return the ``SYSTem:SID:GATeway`` command.

        Description:
            - This command sets or returns the gateway address of the Streaming ID Ethernet port.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:SID:GATeway?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:SID:GATeway?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SYSTem:SID:GATeway value`` command.

        SCPI Syntax:
            ```
            - SYSTem:SID:GATeway <address>
            - SYSTem:SID:GATeway?
            ```
        """
        return self._gateway

    @property
    def ip(self) -> SystemSidIp:
        """Return the ``SYSTem:SID:IP`` command.

        Description:
            - This command sets or returns the IP address of the Streaming ID Ethernet port.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:SID:IP?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:SID:IP?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SYSTem:SID:IP value`` command.

        SCPI Syntax:
            ```
            - SYSTem:SID:IP <address>
            - SYSTem:SID:IP?
            ```
        """
        return self._ip

    @property
    def mac(self) -> SystemSidMac:
        """Return the ``SYSTem:SID:MAC`` command.

        Description:
            - This command returns the MAC address of the Streaming ID port.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:SID:MAC?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:SID:MAC?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:SID:MAC?
            ```
        """
        return self._mac


class SystemErrorNext(SCPICmdRead):
    """The ``SYSTem:ERRor:NEXT`` command.

    Description:
        - This query-only command returns the contents of the Error/Event queue.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:NEXT?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:NEXT?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:ERRor:NEXT?
        ```
    """


class SystemErrorDialog(SCPICmdWrite, SCPICmdRead):
    """The ``SYSTem:ERRor:DIALog`` command.

    Description:
        - This command enables or disables error dialogs from displaying on the UI when an error
          condition occurs on the AWG.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:DIALog?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:DIALog?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SYSTem:ERRor:DIALog value`` command.

    SCPI Syntax:
        ```
        - SYSTem:ERRor:DIALog <show_dialog>
        - SYSTem:ERRor:DIALog?
        ```

    Info:
        - ``*RST`` sets this value to 1.
    """


class SystemErrorCount(SCPICmdRead):
    """The ``SYSTem:ERRor:COUNt`` command.

    Description:
        - This command returns the error and event queue for the number of unread items.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:COUNt?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:COUNt?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:ERRor:COUNt?
        ```
    """


class SystemErrorCodeNext(SCPICmdRead):
    """The ``SYSTem:ERRor:CODE:NEXT`` command.

    Description:
        - This command returns the error and event queue for the next item and removes it from the
          queue.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:CODE:NEXT?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:CODE:NEXT?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:ERRor:CODE:NEXT?
        ```
    """


class SystemErrorCodeAll(SCPICmdRead):
    """The ``SYSTem:ERRor:CODE:ALL`` command.

    Description:
        - This command returns the error and event queue for the codes of all the unread items and
          removes them from the queue.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:CODE:ALL?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:CODE:ALL?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:ERRor:CODE:ALL?
        ```
    """


class SystemErrorCode(SCPICmdRead):
    """The ``SYSTem:ERRor:CODE`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:CODE?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:CODE?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.all``: The ``SYSTem:ERRor:CODE:ALL`` command.
        - ``.next``: The ``SYSTem:ERRor:CODE:NEXT`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._all = SystemErrorCodeAll(device, f"{self._cmd_syntax}:ALL")
        self._next = SystemErrorCodeNext(device, f"{self._cmd_syntax}:NEXT")

    @property
    def all(self) -> SystemErrorCodeAll:
        """Return the ``SYSTem:ERRor:CODE:ALL`` command.

        Description:
            - This command returns the error and event queue for the codes of all the unread items
              and removes them from the queue.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:CODE:ALL?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:CODE:ALL?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:ERRor:CODE:ALL?
            ```
        """
        return self._all

    @property
    def next(self) -> SystemErrorCodeNext:
        """Return the ``SYSTem:ERRor:CODE:NEXT`` command.

        Description:
            - This command returns the error and event queue for the next item and removes it from
              the queue.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:CODE:NEXT?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:CODE:NEXT?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:ERRor:CODE:NEXT?
            ```
        """
        return self._next


class SystemErrorAll(SCPICmdRead):
    """The ``SYSTem:ERRor:ALL`` command.

    Description:
        - This command returns the error and event queue for all the unread items and removes them
          from the queue.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor:ALL?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:ALL?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SYSTem:ERRor:ALL?
        ```
    """


class SystemErrorCmd(SCPICmdRead):
    """The ``SYSTem:ERRor`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:ERRor?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.all``: The ``SYSTem:ERRor:ALL`` command.
        - ``.code``: The ``SYSTem:ERRor:CODE`` command tree.
        - ``.count``: The ``SYSTem:ERRor:COUNt`` command.
        - ``.dialog``: The ``SYSTem:ERRor:DIALog`` command.
        - ``.next``: The ``SYSTem:ERRor:NEXT`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._all = SystemErrorAll(device, f"{self._cmd_syntax}:ALL")
        self._code = SystemErrorCode(device, f"{self._cmd_syntax}:CODE")
        self._count = SystemErrorCount(device, f"{self._cmd_syntax}:COUNt")
        self._dialog = SystemErrorDialog(device, f"{self._cmd_syntax}:DIALog")
        self._next = SystemErrorNext(device, f"{self._cmd_syntax}:NEXT")

    @property
    def all(self) -> SystemErrorAll:
        """Return the ``SYSTem:ERRor:ALL`` command.

        Description:
            - This command returns the error and event queue for all the unread items and removes
              them from the queue.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:ALL?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:ALL?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:ERRor:ALL?
            ```
        """
        return self._all

    @property
    def code(self) -> SystemErrorCode:
        """Return the ``SYSTem:ERRor:CODE`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:CODE?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:CODE?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.all``: The ``SYSTem:ERRor:CODE:ALL`` command.
            - ``.next``: The ``SYSTem:ERRor:CODE:NEXT`` command.
        """
        return self._code

    @property
    def count(self) -> SystemErrorCount:
        """Return the ``SYSTem:ERRor:COUNt`` command.

        Description:
            - This command returns the error and event queue for the number of unread items.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:COUNt?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:COUNt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:ERRor:COUNt?
            ```
        """
        return self._count

    @property
    def dialog(self) -> SystemErrorDialog:
        """Return the ``SYSTem:ERRor:DIALog`` command.

        Description:
            - This command enables or disables error dialogs from displaying on the UI when an error
              condition occurs on the AWG.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:DIALog?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:DIALog?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SYSTem:ERRor:DIALog value``
              command.

        SCPI Syntax:
            ```
            - SYSTem:ERRor:DIALog <show_dialog>
            - SYSTem:ERRor:DIALog?
            ```

        Info:
            - ``*RST`` sets this value to 1.
        """
        return self._dialog

    @property
    def next(self) -> SystemErrorNext:
        """Return the ``SYSTem:ERRor:NEXT`` command.

        Description:
            - This query-only command returns the contents of the Error/Event queue.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor:NEXT?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor:NEXT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:ERRor:NEXT?
            ```
        """
        return self._next


class SystemDate(SCPICmdWrite, SCPICmdRead):
    """The ``SYSTem:DATE`` command.

    Description:
        - This command sets or returns the system date. When the values are nonintegers, they are
          rounded off to nearest integral values.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem:DATE?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:DATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SYSTem:DATE value`` command.

    SCPI Syntax:
        ```
        - SYSTem:DATE <year>,<month>,<day>
        - SYSTem:DATE?
        ```
    """


class System(SCPICmdRead):
    """The ``SYSTem`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SYSTem?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.date``: The ``SYSTem:DATE`` command.
        - ``.error``: The ``SYSTem:ERRor`` command tree.
        - ``.sid``: The ``SYSTem:SID`` command tree.
        - ``.time``: The ``SYSTem:TIME`` command.
        - ``.version``: The ``SYSTem:VERSion`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SYSTem") -> None:
        super().__init__(device, cmd_syntax)
        self._date = SystemDate(device, f"{self._cmd_syntax}:DATE")
        self._error = SystemError(device, f"{self._cmd_syntax}:ERRor")
        self._sid = SystemSid(device, f"{self._cmd_syntax}:SID")
        self._time = SystemTime(device, f"{self._cmd_syntax}:TIME")
        self._version = SystemVersion(device, f"{self._cmd_syntax}:VERSion")

    @property
    def date(self) -> SystemDate:
        """Return the ``SYSTem:DATE`` command.

        Description:
            - This command sets or returns the system date. When the values are nonintegers, they
              are rounded off to nearest integral values.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:DATE?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:DATE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SYSTem:DATE value`` command.

        SCPI Syntax:
            ```
            - SYSTem:DATE <year>,<month>,<day>
            - SYSTem:DATE?
            ```
        """
        return self._date

    @property
    def error(self) -> SystemError:
        """Return the ``SYSTem:ERRor`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:ERRor?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:ERRor?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.all``: The ``SYSTem:ERRor:ALL`` command.
            - ``.code``: The ``SYSTem:ERRor:CODE`` command tree.
            - ``.count``: The ``SYSTem:ERRor:COUNt`` command.
            - ``.dialog``: The ``SYSTem:ERRor:DIALog`` command.
            - ``.next``: The ``SYSTem:ERRor:NEXT`` command.
        """
        return self._error

    @property
    def sid(self) -> SystemSid:
        """Return the ``SYSTem:SID`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:SID?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:SID?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.gateway``: The ``SYSTem:SID:GATeway`` command.
            - ``.ip``: The ``SYSTem:SID:IP`` command.
            - ``.mac``: The ``SYSTem:SID:MAC`` command.
        """
        return self._sid

    @property
    def time(self) -> SystemTime:
        """Return the ``SYSTem:TIME`` command.

        Description:
            - This command sets or returns the system time (hours, minutes and seconds). This
              command is equivalent to the time setting through the Windows Control Panel.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:TIME?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:TIME?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SYSTem:TIME value`` command.

        SCPI Syntax:
            ```
            - SYSTem:TIME <hour>,<minute>,<second>
            - SYSTem:TIME?
            ```
        """
        return self._time

    @property
    def version(self) -> SystemVersion:
        """Return the ``SYSTem:VERSion`` command.

        Description:
            - This query-only command returns the conformed SCPI version of the instrument.

        Usage:
            - Using the ``.query()`` method will send the ``SYSTem:VERSion?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:VERSion?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SYSTem:VERSion?
            ```
        """
        return self._version



"""A class containing properties and attributes shared between devices and control mixins."""

from abc import ABC, abstractmethod
from typing import Tuple

from packaging.version import Version

from tm_devices.helpers import DeviceConfigEntry
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813


class _AbstractDeviceControl(ABC):  # pyright: ignore[reportUnusedClass]
    """Abstract class with properties and attributes shared between devices and control mixins."""

    def __init__(self, config_entry: DeviceConfigEntry, verbose: bool) -> None:
        """Create a device.

        Args:
            config_entry: A config entry object parsed by the DMConfigParser.
            verbose: A boolean indicating if verbose output should be printed. If True,
                communication printouts will be logged with a level of INFO. If False,
                communication printouts will be logged with a level of DEBUG.
        """
        self._is_open = False
        self._verbose = verbose
        self._config_entry = config_entry
        self._enable_verification = True

    @cached_property
    @abstractmethod
    def name_and_alias(self) -> str:
        """A string containing the device name and alias."""

    @cached_property
    @abstractmethod
    def ip_address(self) -> str:
        """The IP address of the device."""

    @abstractmethod
    def _get_errors(self) -> Tuple[int, Tuple[str, ...]]:
        """Get the current errors from the device.

        !!! note
            This method will clear out the error queue after reading the current errors.

        Returns:
            A tuple containing the current error code alongside a tuple of the current error
                messages.
        """

    @cached_property
    @abstractmethod
    def device_type(self) -> str:
        """Return a string representing the device type."""

    @cached_property
    @abstractmethod
    def manufacturer(self) -> str:
        """Return the manufacturer of the device."""

    @cached_property
    @abstractmethod
    def model(self) -> str:
        """Return the full model of the device."""

    @cached_property
    @abstractmethod
    def serial(self) -> str:
        """Return the serial number of the device."""

    @cached_property
    @abstractmethod
    def sw_version(self) -> Version:
        """Return the software version of the device."""

    @abstractmethod
    def _cleanup(self) -> None:
        """Perform the actual cleanup code."""

    @abstractmethod
    def _close(self) -> None:
        """Perform the actual closing code."""

    @abstractmethod
    def _open(self) -> bool:
        """Perform the actual opening code.

        Returns:
           A boolean indicating if device connected successfully.
        """

    def _reboot(self) -> None:
        """Perform the actual rebooting code."""
        raise NotImplementedError(
            f"``._reboot()`` is not yet implemented for the {self.__class__.__name__} driver",  # noqa: EM102
        )



"""The gpibusb commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - GPIBUsb:ID?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class GpibusbId(SCPICmdRead):
    """The ``GPIBUsb:ID`` command.

    Description:
        - Returns the identification string of the connected TEK-USB-488 adaptor option and firmware
          version. If a TEK-USB-488.2 option is not connected, the system returns 'Not detected'.

    Usage:
        - Using the ``.query()`` method will send the ``GPIBUsb:ID?`` query.
        - Using the ``.verify(value)`` method will send the ``GPIBUsb:ID?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - GPIBUsb:ID?
        ```
    """


class Gpibusb(SCPICmdRead):
    """The ``GPIBUsb`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``GPIBUsb?`` query.
        - Using the ``.verify(value)`` method will send the ``GPIBUsb?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.id``: The ``GPIBUsb:ID`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "GPIBUsb") -> None:
        super().__init__(device, cmd_syntax)
        self._id = GpibusbId(device, f"{self._cmd_syntax}:ID")

    @property
    def id(self) -> GpibusbId:
        """Return the ``GPIBUsb:ID`` command.

        Description:
            - Returns the identification string of the connected TEK-USB-488 adaptor option and
              firmware version. If a TEK-USB-488.2 option is not connected, the system returns 'Not
              detected'.

        Usage:
            - Using the ``.query()`` method will send the ``GPIBUsb:ID?`` query.
            - Using the ``.verify(value)`` method will send the ``GPIBUsb:ID?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - GPIBUsb:ID?
            ```
        """
        return self._id



"""The date commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC,
DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DATE <QString>
    - DATE?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Date(SCPICmdWrite, SCPICmdRead):
    """The ``DATE`` command.

    Description:
        - This command specifies the date the oscilloscope displays.

    Usage:
        - Using the ``.query()`` method will send the ``DATE?`` query.
        - Using the ``.verify(value)`` method will send the ``DATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATE value`` command.

    SCPI Syntax:
        ```
        - DATE <QString>
        - DATE?
        ```

    Info:
        - ``<QString>`` is a date in the form 'yyyy-mm-dd' where yyyy refers to a four-digit year
          number, mm refers to a two-digit month number from 01 to 12, and dd refers to a two-digit
          day number in the month.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DATE") -> None:
        super().__init__(device, cmd_syntax)



"""The mch commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MCH<x>:MAXAMPLitude <NR3>
    - MCH<x>:MINAMPLitude <NR3>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MchItemMinamplitude(SCPICmdWrite):
    """The ``MCH<x>:MINAMPLitude`` command.

    Description:
        - These commands set or query the minimum/maximum amplitude vertical setting for the
          unmapped channels on an UltraSync stack master oscilloscope. These two commands are used
          together to set the vertical amplitude range for the unmapped channels on an UltraSync
          stack master. The vertical system scales the unmapped channel so that a large signal (that
          is not clipped, 6 to 8 divisions high centered about ground or equivalent) is available to
          the trigger system. It is important to match these min/max amplitude settings to the
          actual signal under test. Briefly, in a stack of oscilloscopes, the mapped channels are
          those used to acquire data (one channel from each oscilloscope in the stack). The mapped
          channels on one of the oscilloscopes in a stack can also be used for triggering. The
          unmapped channels on a stack master are also available for triggering. These channels are
          not acquired, so no waveforms for these channels are displayed. These channels are
          designated MCH1, MCH2, MCH3, and MCH4. On an ATI stack, CH2, MCH1, and MCH3 are available
          for triggering. On a 4-channel stack, CH1, MCH2, MCH3, and MCH4 are available for
          triggering. A more detailed discussion follows. Mapped Channels In a stack configuration
          of oscilloscopes, one channel from each extension is mapped to the master along with a
          single channel on the master. The waveforms acquired from these channels on the respective
          oscilloscopes are gathered together for display on the master oscilloscope. These are the
          mapped channels. Unmapped Channels In a stack configuration of oscilloscopes, the master
          has additional channels that are not used for acquiring waveforms, but they can be used to
          trigger the oscilloscope. These channels are called the unmapped channels, and they are
          only available on the master of an UltraSync stack. These unmapped channels on the master
          have their own vertical user interface control tab, where you enter the expected minimum
          and maximum amplitude values for the device under test signals. When you set the min/max
          amplitude values, the oscilloscope hardware is configured so that the device under test
          signals present at the channel probe tip (or input connector) will provide the largest
          possible trigger stimulus without clipping or other nonlinearities. Note that waveforms
          from these unmapped channels are not acquired, and are thus not visible on the
          oscilloscope display. This means you must find a way to quantify the min/max amplitude of
          the device under test signals. One method is to temporarily take the master oscilloscope
          out of the stack (that is, go to standalone mode) to determine the best values for the
          min/max amplitudes of the device under test signals, and then put the master oscilloscope
          back into the UltraSync stack where you then set the vertical min/max amplitude for the
          unmapped channels. These non-acquired, but useful for triggering, unmapped channels on the
          master in an UltraSync stack are referenced by the terms MCH1, MCH2, MCH3, and MCH4 on the
          Vertical menu. Case 1 UltraSync ATI stack in ATI mode: In an ATI stack in ATI mode, all
          ATI channels in the stack are mapped to the master. Waveforms are acquired and displayed
          for all of these ATI channels. The non-acquired unmapped channels on the master are
          available as sources for triggering, and are referenced as MCH1 and MCH3. Case 2 UltraSync
          ATI stack in TekConnect mode: The only unmapped channel on the master is MCh2 (the ATI
          channel). It may be a trigger source. Case 3 UltraSync non-ATI stack (a stack of 4-channel
          oscilloscopes). The unmapped channels on the stack master, MCH2, MCH3, and MCH4, may be
          trigger sources. Case 4 Time-Sync stack. All channels on the master are mapped channels,
          and so are always available as trigger sources. Case 5 Standalone mode (single
          oscilloscope). All channels are mapped channels, and so are always available as trigger
          sources.

    Usage:
        - Using the ``.write(value)`` method will send the ``MCH<x>:MINAMPLitude value`` command.

    SCPI Syntax:
        ```
        - MCH<x>:MINAMPLitude <NR3>
        ```

    Info:
        - ``<NR3>`` the minimum/maximum amplitude vertical setting for the unmapped channels on an
          UltraSync stack master oscilloscope.
    """


class MchItemMaxamplitude(SCPICmdWrite):
    """The ``MCH<x>:MAXAMPLitude`` command.

    Description:
        - These commands set or query the minimum/maximum amplitude vertical setting for the
          unmapped channels on an UltraSync stack master oscilloscope. These two commands are used
          together to set the vertical amplitude range for the unmapped channels on an UltraSync
          stack master. The vertical system scales the unmapped channel so that a large signal (that
          is not clipped, 6 to 8 divisions high centered about ground or equivalent) is available to
          the trigger system. It is important to match these min/max amplitude settings to the
          actual signal under test. Briefly, in a stack of oscilloscopes, the mapped channels are
          those used to acquire data (one channel from each oscilloscope in the stack). The mapped
          channels on one of the oscilloscopes in a stack can also be used for triggering. The
          unmapped channels on a stack master are also available for triggering. These channels are
          not acquired, so no waveforms for these channels are displayed. These channels are
          designated MCH1, MCH2, MCH3, and MCH4. On an ATI stack, CH2, MCH1, and MCH3 are available
          for triggering. On a 4-channel stack, CH1, MCH2, MCH3, and MCH4 are available for
          triggering. A more detailed discussion follows. Mapped Channels In a stack configuration
          of oscilloscopes, one channel from each extension is mapped to the master along with a
          single channel on the master. The waveforms acquired from these channels on the respective
          oscilloscopes are gathered together for display on the master oscilloscope. These are the
          mapped channels. Unmapped Channels In a stack configuration of oscilloscopes, the master
          has additional channels that are not used for acquiring waveforms, but they can be used to
          trigger the oscilloscope. These channels are called the unmapped channels, and they are
          only available on the master of an UltraSync stack. These unmapped channels on the master
          have their own vertical user interface control tab, where you enter the expected minimum
          and maximum amplitude values for the device under test signals. When you set the min/max
          amplitude values, the oscilloscope hardware is configured so that the device under test
          signals present at the channel probe tip (or input connector) will provide the largest
          possible trigger stimulus without clipping or other nonlinearities. Note that waveforms
          from these unmapped channels are not acquired, and are thus not visible on the
          oscilloscope display. This means you must find a way to quantify the min/max amplitude of
          the device under test signals. One method is to temporarily take the master oscilloscope
          out of the stack (that is, go to standalone mode) to determine the best values for the
          min/max amplitudes of the device under test signals, and then put the master oscilloscope
          back into the UltraSync stack where you then set the vertical min/max amplitude for the
          unmapped channels. These non-acquired, but useful for triggering, unmapped channels on the
          master in an UltraSync stack are referenced by the terms MCH1, MCH2, MCH3, and MCH4 on the
          Vertical menu. Case 1 UltraSync ATI stack in ATI mode: In an ATI stack in ATI mode, all
          ATI channels in the stack are mapped to the master. Waveforms are acquired and displayed
          for all of these ATI channels. The non-acquired unmapped channels on the master are
          available as sources for triggering, and are referenced as MCH1 and MCH3. Case 2 UltraSync
          ATI stack in TekConnect mode: The only unmapped channel on the master is MCh2 (the ATI
          channel). It may be a trigger source. Case 3 UltraSync non-ATI stack (a stack of 4-channel
          oscilloscopes). The unmapped channels on the stack master, MCH2, MCH3, and MCH4, may be
          trigger sources. Case 4 Time-Sync stack. All channels on the master are mapped channels,
          and so are always available as trigger sources. Case 5 Standalone mode (single
          oscilloscope). All channels are mapped channels, and so are always available as trigger
          sources.

    Usage:
        - Using the ``.write(value)`` method will send the ``MCH<x>:MAXAMPLitude value`` command.

    SCPI Syntax:
        ```
        - MCH<x>:MAXAMPLitude <NR3>
        ```

    Info:
        - ``<NR3>`` the minimum/maximum amplitude vertical setting for the unmapped channels on an
          UltraSync stack master oscilloscope.
    """


class MchItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``MCH<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MCH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``MCH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.minamplitude``: The ``MCH<x>:MINAMPLitude`` command.
        - ``.maxamplitude``: The ``MCH<x>:MAXAMPLitude`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MCH<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._minamplitude = MchItemMinamplitude(device, f"{self._cmd_syntax}:MINAMPLitude")
        self._maxamplitude = MchItemMaxamplitude(device, f"{self._cmd_syntax}:MAXAMPLitude")

    @property
    def minamplitude(self) -> MchItemMinamplitude:
        """Return the ``MCH<x>:MINAMPLitude`` command.

        Description:
            - These commands set or query the minimum/maximum amplitude vertical setting for the
              unmapped channels on an UltraSync stack master oscilloscope. These two commands are
              used together to set the vertical amplitude range for the unmapped channels on an
              UltraSync stack master. The vertical system scales the unmapped channel so that a
              large signal (that is not clipped, 6 to 8 divisions high centered about ground or
              equivalent) is available to the trigger system. It is important to match these min/max
              amplitude settings to the actual signal under test. Briefly, in a stack of
              oscilloscopes, the mapped channels are those used to acquire data (one channel from
              each oscilloscope in the stack). The mapped channels on one of the oscilloscopes in a
              stack can also be used for triggering. The unmapped channels on a stack master are
              also available for triggering. These channels are not acquired, so no waveforms for
              these channels are displayed. These channels are designated MCH1, MCH2, MCH3, and
              MCH4. On an ATI stack, CH2, MCH1, and MCH3 are available for triggering. On a
              4-channel stack, CH1, MCH2, MCH3, and MCH4 are available for triggering. A more
              detailed discussion follows. Mapped Channels In a stack configuration of
              oscilloscopes, one channel from each extension is mapped to the master along with a
              single channel on the master. The waveforms acquired from these channels on the
              respective oscilloscopes are gathered together for display on the master oscilloscope.
              These are the mapped channels. Unmapped Channels In a stack configuration of
              oscilloscopes, the master has additional channels that are not used for acquiring
              waveforms, but they can be used to trigger the oscilloscope. These channels are called
              the unmapped channels, and they are only available on the master of an UltraSync
              stack. These unmapped channels on the master have their own vertical user interface
              control tab, where you enter the expected minimum and maximum amplitude values for the
              device under test signals. When you set the min/max amplitude values, the oscilloscope
              hardware is configured so that the device under test signals present at the channel
              probe tip (or input connector) will provide the largest possible trigger stimulus
              without clipping or other nonlinearities. Note that waveforms from these unmapped
              channels are not acquired, and are thus not visible on the oscilloscope display. This
              means you must find a way to quantify the min/max amplitude of the device under test
              signals. One method is to temporarily take the master oscilloscope out of the stack
              (that is, go to standalone mode) to determine the best values for the min/max
              amplitudes of the device under test signals, and then put the master oscilloscope back
              into the UltraSync stack where you then set the vertical min/max amplitude for the
              unmapped channels. These non-acquired, but useful for triggering, unmapped channels on
              the master in an UltraSync stack are referenced by the terms MCH1, MCH2, MCH3, and
              MCH4 on the Vertical menu. Case 1 UltraSync ATI stack in ATI mode: In an ATI stack in
              ATI mode, all ATI channels in the stack are mapped to the master. Waveforms are
              acquired and displayed for all of these ATI channels. The non-acquired unmapped
              channels on the master are available as sources for triggering, and are referenced as
              MCH1 and MCH3. Case 2 UltraSync ATI stack in TekConnect mode: The only unmapped
              channel on the master is MCh2 (the ATI channel). It may be a trigger source. Case 3
              UltraSync non-ATI stack (a stack of 4-channel oscilloscopes). The unmapped channels on
              the stack master, MCH2, MCH3, and MCH4, may be trigger sources. Case 4 Time-Sync
              stack. All channels on the master are mapped channels, and so are always available as
              trigger sources. Case 5 Standalone mode (single oscilloscope). All channels are mapped
              channels, and so are always available as trigger sources.

        Usage:
            - Using the ``.write(value)`` method will send the ``MCH<x>:MINAMPLitude value``
              command.

        SCPI Syntax:
            ```
            - MCH<x>:MINAMPLitude <NR3>
            ```

        Info:
            - ``<NR3>`` the minimum/maximum amplitude vertical setting for the unmapped channels on
              an UltraSync stack master oscilloscope.
        """
        return self._minamplitude

    @property
    def maxamplitude(self) -> MchItemMaxamplitude:
        """Return the ``MCH<x>:MAXAMPLitude`` command.

        Description:
            - These commands set or query the minimum/maximum amplitude vertical setting for the
              unmapped channels on an UltraSync stack master oscilloscope. These two commands are
              used together to set the vertical amplitude range for the unmapped channels on an
              UltraSync stack master. The vertical system scales the unmapped channel so that a
              large signal (that is not clipped, 6 to 8 divisions high centered about ground or
              equivalent) is available to the trigger system. It is important to match these min/max
              amplitude settings to the actual signal under test. Briefly, in a stack of
              oscilloscopes, the mapped channels are those used to acquire data (one channel from
              each oscilloscope in the stack). The mapped channels on one of the oscilloscopes in a
              stack can also be used for triggering. The unmapped channels on a stack master are
              also available for triggering. These channels are not acquired, so no waveforms for
              these channels are displayed. These channels are designated MCH1, MCH2, MCH3, and
              MCH4. On an ATI stack, CH2, MCH1, and MCH3 are available for triggering. On a
              4-channel stack, CH1, MCH2, MCH3, and MCH4 are available for triggering. A more
              detailed discussion follows. Mapped Channels In a stack configuration of
              oscilloscopes, one channel from each extension is mapped to the master along with a
              single channel on the master. The waveforms acquired from these channels on the
              respective oscilloscopes are gathered together for display on the master oscilloscope.
              These are the mapped channels. Unmapped Channels In a stack configuration of
              oscilloscopes, the master has additional channels that are not used for acquiring
              waveforms, but they can be used to trigger the oscilloscope. These channels are called
              the unmapped channels, and they are only available on the master of an UltraSync
              stack. These unmapped channels on the master have their own vertical user interface
              control tab, where you enter the expected minimum and maximum amplitude values for the
              device under test signals. When you set the min/max amplitude values, the oscilloscope
              hardware is configured so that the device under test signals present at the channel
              probe tip (or input connector) will provide the largest possible trigger stimulus
              without clipping or other nonlinearities. Note that waveforms from these unmapped
              channels are not acquired, and are thus not visible on the oscilloscope display. This
              means you must find a way to quantify the min/max amplitude of the device under test
              signals. One method is to temporarily take the master oscilloscope out of the stack
              (that is, go to standalone mode) to determine the best values for the min/max
              amplitudes of the device under test signals, and then put the master oscilloscope back
              into the UltraSync stack where you then set the vertical min/max amplitude for the
              unmapped channels. These non-acquired, but useful for triggering, unmapped channels on
              the master in an UltraSync stack are referenced by the terms MCH1, MCH2, MCH3, and
              MCH4 on the Vertical menu. Case 1 UltraSync ATI stack in ATI mode: In an ATI stack in
              ATI mode, all ATI channels in the stack are mapped to the master. Waveforms are
              acquired and displayed for all of these ATI channels. The non-acquired unmapped
              channels on the master are available as sources for triggering, and are referenced as
              MCH1 and MCH3. Case 2 UltraSync ATI stack in TekConnect mode: The only unmapped
              channel on the master is MCh2 (the ATI channel). It may be a trigger source. Case 3
              UltraSync non-ATI stack (a stack of 4-channel oscilloscopes). The unmapped channels on
              the stack master, MCH2, MCH3, and MCH4, may be trigger sources. Case 4 Time-Sync
              stack. All channels on the master are mapped channels, and so are always available as
              trigger sources. Case 5 Standalone mode (single oscilloscope). All channels are mapped
              channels, and so are always available as trigger sources.

        Usage:
            - Using the ``.write(value)`` method will send the ``MCH<x>:MAXAMPLitude value``
              command.

        SCPI Syntax:
            ```
            - MCH<x>:MAXAMPLitude <NR3>
            ```

        Info:
            - ``<NR3>`` the minimum/maximum amplitude vertical setting for the unmapped channels on
              an UltraSync stack master oscilloscope.
        """
        return self._maxamplitude
